/*
 * File : window.h
 *
 * Description : This file contains the declaration of the 
 * window class.
 */

#ifndef _window_h_
#define _window_h_


#include "frame.h"

class dllayer;

/*
 * Class : window
 *
 * Description : This class provides the functionality for
 * implementing sliding-window protocols.
 * The class can verify N(r) values and  N(s) values in 
 * received frames, form I-frames for transmission, check
 * for window FULL conditions and manage the state variables
 * V(s) and V(r).
 * Provides convenience functions to retrieve information
 * about the state of the window.
 * 
 * Attributes:
 *
 *	int	_modulus: The modulus of the sliding window.
 *	int	_vs: The send state variable V(s)
 *	int	_vr: The recv state variable V(r)
 *	int	_ackPending: The number of acknowledgements pending.
 *	frameList 	_xmitList: A queue of I-frames which have already been
 *		transmitted and pending ack, or have to be transmitted.
 */
 

class window {

 public:

	window(int modulus);
	virtual ~window();

	dataPacket *GetNextToXmit();
	int ProcessAck(int nr);
	void AddFrame(dataPacket *dp);
	Boolean CheckSequence(int ns);
		
	int GetVr() { return _vr; } 	
	int GetVs() { return _vs; }
	int UnackedFrames() { return _ackPending; }
	int GetQueueLength() { return _xmitList.Length(); }
	void Reset();
	void StartRetransmission();

 private:
	void UnqueueFrames(int num);
	int		_modulus;
	int		_vs;
	int		_vr;
	int		_ackPending;
	frameList 	_xmitList;
	
};


#endif
